<?php

use JsonSchema\Validator;
use Zen\Payment\Util;

/**
 * Class ControllerExtensionPaymentZen
 */
class ControllerExtensionPaymentZen extends Controller
{
    private $error = [];

    /**
     * view admin panel
     */
    public function index()
    {
        $this->language->load('extension/payment/zen');
        $this->document->setTitle($this->language->get('heading_title'));
        $this->document->addScript('view/javascript/select2/js/select2.min.js');
        $this->document->addStyle('view/javascript/select2/css/select2.css');
        $this->load->model('setting/setting');

        $request = $this->request->post;

        if (($this->request->server['REQUEST_METHOD'] == 'POST')
            && $this->validate($request)
        ) {

            $settings = array_merge($this->model_setting_setting->getSetting('payment_zen'), $request);
            $this->model_setting_setting->editSetting('payment_zen', $settings);
            $this->session->data['success'] = $this->language->get('text_success');
            $this->response->redirect($this->url->link('marketplace/extension', 'user_token=' . $this->session->data['user_token'] . '&type=payment', true));

            return;
        }

        include_once(DIR_SYSTEM . 'library/payment-core/vendor/autoload.php');

        $data = [

            'zen_status' => $this->request->post['payment_zen_status'] ?? $this->config->get('payment_zen_status'),
            'zen_terminal_uuid' => $this->request->post['payment_zen_terminal_uuid'] ?? $this->config->get('payment_zen_terminal_uuid'),
            'zen_paywall_secret' => $this->request->post['payment_zen_paywall_secret'] ?? $this->config->get('payment_zen_paywall_secret'),
            'zen_ipn_secret' => $this->request->post['payment_zen_ipn_secret'] ?? $this->config->get('payment_zen_ipn_secret'),
            'zen_sort_order' => $this->request->post['payment_zen_sort_order'] ?? $this->config->get('payment_zen_sort_order'),
            'zen_payment_title_default' => $this->language->get('text_zen_payment_title_default'),
            'zen_payment_title' => isset($this->request->post['payment_zen_payment_title'])
                ? $this->request->post['payment_zen_sort_order']
                : $this->config->get('payment_zen_payment_title'),
            'zen_all_currencies' => Util::getCurrencies(),
            'zen_currencies' => $this->request->post['payment_zen_currencies'] ?? $this->config->get('payment_zen_currencies'),
            'error_validation' => $this->error,
            'notification_url' => HTTPS_CATALOG . 'index.php?route=extension/payment/zen/notification',
            'confirm_url' => $this->url->link('extension/payment/zen', 'user_token=' . $this->session->data['user_token'], true),
            'cancel_url' => $this->url->link('extension/extension', 'user_token=' . $this->session->data['user_token'], true),
            'content_footer' => $this->load->controller('common/footer'),
            'content_header' => $this->load->controller('common/header'),
            'content_column_left' => $this->load->controller('common/column_left'),
            'heading_title' => $this->language->get('text_heading_title'),
            'breadcrumbs' => [
                [
                    'text' => $this->language->get('text_home'),
                    'href' => $this->url->link('common/home', 'user_token=' . $this->session->data['user_token'], 'SSL'),
                    'separator' => false,
                ],

                [
                    'text' => $this->language->get('text_payment'),
                    'href' => $this->url->link('extension/extension', 'user_token=' . $this->session->data['user_token'], 'SSL'),
                    'separator' => ' :: ',
                ],

                [
                    'text' => $this->language->get('heading_title'),
                    'href' => $this->url->link('extension/payment/zen', 'user_token=' . $this->session->data['user_token'], 'SSL'),
                    'separator' => ' :: ',
                ],
            ],

        ];

        $this->response->setOutput($this->load->view('extension/payment/zen', $data));
    }

    /**
     * @param array $request
     *
     * @return bool
     */
    protected function validate($request)
    {
        if (!$this->user->hasPermission('modify', 'extension/payment/zen')) {
            $this->error['permission'] = true;

            return false;
        }

        include_once(DIR_SYSTEM . 'library/payment-core/vendor/autoload.php');

        $schema = [
            'type' => 'object',
            'properties' => [

                'payment_zen_status' => [
                    'type' => 'string',
                    "enum" => [
                        '0',
                        '1',
                    ],
                ],
                'payment_zen_terminal_uuid' => [
                    'type' => 'string',
                ],
                'payment_zen_paywall_secret' => [
                    'type' => 'string',
                ],
                'payment_zen_ipn_secret' => [
                    'type' => 'string',
                ],
                'payment_zen_sort_order' => [
                    'type' => 'string',
                ],
                'payment_zen_payment_title' => [
                    'type' => 'string',
                ],
                'payment_zen_currencies' => [
                    'type' => 'array',
                    'items' => [
                        'type' => 'string',
                        'enum' => array_values(Util::getCurrencies()),
                    ],
                    'minItems' => 1,
                    'uniqueItems' => true,
                ]
            ],

            'required' => [
                'payment_zen_status',
                'payment_zen_terminal_uuid',
                'payment_zen_paywall_secret',
                'payment_zen_ipn_secret',
                'payment_zen_sort_order',
                'payment_zen_payment_title',
                'payment_zen_currencies',
            ],
            'additionalProperties' => false,
        ];

        $data = json_decode(json_encode($request));

        $validator = new Validator();
        $validator->validate($data, $schema);

        $this->error = [];

        foreach ($validator->getErrors() as $value) {

            if ($value['constraint'] === 'additionalProp') {
                $this->error['additional_properties'] = true;
            } else {
                $this->error[$value['property']] = true;
            }
        }

        return empty($this->error);
    }
}
